<?php

namespace BULKYWP\Includes;

defined( 'ABSPATH' ) || exit;

class Data_Comment {

	protected static $instance = null;

	protected static $params;
	protected $filter;
	protected $settings;

	public function __construct() {
	}

	public static function instance() {
		return self::$instance == null ? self::$instance = new self : self::$instance;
	}

	public static function get_params() {
		$params_saved   = get_option( 'wpbulky_params_comment_setting' );
		$default_params = [

		];

		return self::$params = wp_parse_args( $params_saved, $default_params );
	}

	public static function get_param( $key ) {
		if ( ! self::$params ) {
			self::get_params();
		}

		return self::$params[ $key ] ?? '';
	}

	public function fixed_columns() {
		return [ 'id' ];
	}

	public function define_columns_type() {

		$columns = [
			'id'      => [ 'type' => 'number', 'width' => 70, 'title' => 'ID', 'readOnly' => true ],
			'post_id' => [
				'type'     => 'number',
				'width'    => 100,
				'title'    => esc_html__( 'Post ID', 'wpbulky-wordpress-bulk-edit-post-types' ),
				'readOnly' => true,
			],

			'comment_type' => [
				'type'     => 'text',
				'width'    => 70,
				'title'    => esc_html__( 'Type', 'wpbulky-wordpress-bulk-edit-post-types' ),
				'readOnly' => true,
			],

			'content' => [
				'type'   => 'custom',
				'width'  => 120,
				'title'  => esc_html__( 'Content', 'wpbulky-wordpress-bulk-edit-post-types' ),
				'align'  => 'left',
				'editor' => 'textEditor',
			],

			'author_name' => [
				'type'  => 'text',
				'width' => 100,
				'title' => esc_html__( 'Author Name', 'wpbulky-wordpress-bulk-edit-post-types' ),
				'align' => 'left',
			],

			'author_email' => [
				'type'  => 'text',
				'width' => 100,
				'title' => esc_html__( 'Author Email', 'wpbulky-wordpress-bulk-edit-post-types' ),
				'align' => 'left',
			],

			'author_url' => [
				'type'  => 'text',
				'width' => 100,
				'title' => esc_html__( 'Author URL', 'wpbulky-wordpress-bulk-edit-post-types' ),
				'align' => 'left',
			],

			'status' => [
				'type'      => 'dropdown',
				'width'     => 80,
				'title'     => esc_html__( 'Status', 'wpbulky-wordpress-bulk-edit-post-types' ),
				'source'    => $this->parse_to_dropdown_source( [
					'approve' => 'Approved',
					'pending' => 'Pending',
					'spam'    => 'Spam'
				] ),
				'subSource' => [
					[ 'id' => 'publish', 'name' => esc_html__( 'Enable', 'wpbulky-wordpress-bulk-edit-post-types' ) ],
					[ 'id' => 'private', 'name' => esc_html__( 'Disable', 'wpbulky-wordpress-bulk-edit-post-types' ) ],
				],
				'filter'    => 'sourceForVariation'
			],

			'comment_date' => [
				'type'    => 'calendar',
				'width'   => 120,
				'title'   => esc_html__( 'Comment Date', 'wpbulky-wordpress-bulk-edit-post-types' ),
				'options' => [ 'format' => 'YYYY-MM-DD HH24:MI', 'time' => 1 ],
			],

		];


		$meta_fields = get_option( "vi_wpbulky_comment_meta_fields" );

		$meta_field_columns = [];
		if ( ! empty( $meta_fields ) && is_array( $meta_fields ) ) {
			foreach ( $meta_fields as $meta_key => $meta_field ) {
				if ( empty( $meta_field['active'] ) ) {
					continue;
				}

				$multiple = $meta_field['multiple'] ?? false;
				$type     = 'text';
				$editor   = '';

				if ( $multiple ) {
					$meta_field['input_type'] = 'array';
				}

				switch ( $meta_field['input_type'] ) {
					case 'textinput':
						$type = 'text';
						break;

					case 'numberinput':
						$type = 'number';
						break;

					case 'checkbox':
						$type = 'checkbox';
						break;

					case 'array':
					case 'json':
						$type   = 'custom';
						$editor = 'array';
						break;

					case 'calendar':
						$type = 'calendar';
						break;

					case 'texteditor':
						$type   = 'custom';
						$editor = 'textEditor';
						break;

					case 'image':
						$type   = 'custom';
						$editor = 'image';
						break;

					case 'imageurl':
						$type   = 'custom';
						$editor = 'imageurl';
						break;
				}

				$meta_field_columns[ $meta_key ] = [
					'title'  => ! empty( $meta_field['column_name'] ) ? $meta_field['column_name'] : $meta_key,
					'width'  => 100,
					'type'   => $type,
					'editor' => $editor,
				];

			}
		}

		$columns = array_merge( $columns, $meta_field_columns );

		if ( ! empty( $columns ) && is_array( $columns ) ) {
			foreach ( $columns as $col_id => $column ) {
				$columns[ $col_id ] = apply_filters( "wpbulky_comment_define_column", $column, $col_id );
			}
		}
		
		$comment_column_width = get_option( "vi_wpbulky_comment_column_width", [] );

		if ( empty( $comment_column_width ) ) {
			$comment_column_width = [];
			foreach ( $columns as $col_id => $column ) {
				$comment_column_width[ $col_id ] = $column['width'];
			}
			update_option( "vi_wpbulky_comment_column_width", $comment_column_width );

		}else {
			foreach ( $columns as $col_id => $column ) {
				if ( isset( $comment_column_width[ $col_id ] ) ) {
					$columns[$col_id]['width'] = $comment_column_width[ $col_id ];
				}
			}
		}

		return $columns;
	}

	public function parse_to_dropdown_source( $options ) {
		$r = [];
		if ( ! empty( $options ) && is_array( $options ) ) {
			foreach ( $options as $id => $name ) {
				$r[] = [ 'id' => $id, 'name' => $name ];
			}
		}

		return $r;
	}

	public function get_culumns_type_title() {
		$columns = wp_list_pluck( $this->define_columns_type(), 'title' );
		unset( $columns['id'] );

		return $columns;
	}

	public function get_settings() {
		if ( ! $this->settings ) {
			global $wpbulky_post_type;
			$this->settings = wp_parse_args(
				get_option( "vi_wpbulky_comment_settings", [] ),
				[
					'edit_fields'          => [],
					'comments_per_page'    => 10,
					'order_by'             => 'ID',
					'order'                => 'DESC',
					'auto_save_revision'   => 60,
					'auto_remove_revision' => 30,
				]
			);
		}

		return $this->settings;
	}

	public function get_setting( $key ) {
		$all_settings = $this->get_settings();

		return $all_settings[ $key ] ?? '';
	}

	public function get_fields_for_parse_post() {
		$defined_columns     = array_keys( $this->define_columns_type() );
		$edit_fields         = $this->get_setting( 'edit_fields' );
		$exclude_edit_fields = $this->get_setting( 'exclude_edit_fields' );

		$r = $defined_columns;

		if ( ! empty( $edit_fields ) && is_array( $edit_fields ) ) {
			$edit_fields = array_merge( $this->fixed_columns(), $edit_fields );

			foreach ( $r as $i => $key ) { //Keep piority
				if ( $key !== false && ! in_array( $key, $edit_fields ) ) {
					unset( $r[ $i ] );
				}
			}
		}

		if ( ! empty( $exclude_edit_fields ) && is_array( $exclude_edit_fields ) ) {
			foreach ( $exclude_edit_fields as $field ) {
				$key = array_search( $field, $r );

				if ( $key !== false && isset( $r[ $key ] ) ) {
					unset( $r[ $key ] );
				}
			}
		}

		return array_values( $r );
	}

	public function get_columns_type() {
		$columns          = $this->define_columns_type();
		$accepted_columns = [];
		$patterns         = $this->get_fields_for_parse_post();

		if ( ! empty( $patterns ) ) {
			foreach ( $columns as $key => $column ) {
				if ( in_array( $key, $patterns ) ) {
					$column['id']             = $key;
					$accepted_columns[ $key ] = $column;
				}
			}
		} else {
			$accepted_columns = $columns;
		}

		return $accepted_columns;
	}

	public function isHTML( $string ) {
		return $string != wp_strip_all_tags( $string ) ? true : false;
	}

	public function sanitize( $var ) {
		if ( is_array( $var ) ) {
			return array_map( [ $this, 'sanitize' ], $var );
		} elseif ( $this->isHTML( $var ) ) {
			return wp_kses_post( $var );
		} else {
			return is_scalar( $var ) ? sanitize_text_field( $var ) : $var;
		}
	}
}